package com.pj.biz;

import java.util.List;
import java.util.stream.Collectors;

import cn.hutool.core.util.StrUtil;
import com.pj.project.tb_person_visit_record.TbPersonVisitRecord;
import com.pj.project.tb_person_visit_record.TbPersonVisitRecordService;
import com.pj.project.tb_person_visit_record.VisitAddBO;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.*;

import com.pj.utils.sg.*;
import com.pj.utils.so.*;
import com.pj.project4sp.SP;

import com.pj.current.satoken.StpUserUtil;
import cn.dev33.satoken.annotation.SaCheckPermission;


/**
 * Controller: tb_person_visit_record -- 人员出入记录
 *
 * @author qzyReal
 */
@RestController
@RequestMapping("/TbPersonVisitRecord/")
public class TbPersonVisitRecordController {

    /**
     * 底层 Service 对象
     */
    @Autowired
    TbPersonVisitRecordService tbPersonVisitRecordService;

    /**
     * 增
     */
    @RequestMapping("add")
    @SaCheckPermission(TbPersonVisitRecord.PERMISSION_CODE_ADD)
    public AjaxJson add(VisitAddBO t) {
        tbPersonVisitRecordService.add(t);
        return AjaxJson.getSuccessData(t);
    }
    /**
     * 健康码记录-增
     */
    @RequestMapping("addHealthRecord")
    @SaCheckPermission(TbPersonVisitRecord.PERMISSION_CODE_HEALTH_ADD)
    public AjaxJson addHealthRecord(VisitAddBO t) {
        tbPersonVisitRecordService.add(t);
        return AjaxJson.getSuccessData(t);
    }

    /**
     * 删
     */
    @RequestMapping("delete")
    @SaCheckPermission(TbPersonVisitRecord.PERMISSION_CODE)
    public AjaxJson delete(Long id) {
        tbPersonVisitRecordService.delete(id);
        return AjaxJson.getSuccess();
    }
    /**
     * 健康码记录-删除
     */
    @RequestMapping("deleteHealthRecord")
    @SaCheckPermission(TbPersonVisitRecord.PERMISSION_CODE_HEALTH_DELETE)
    public AjaxJson deleteHealthRecord(Long id) {
        tbPersonVisitRecordService.delete(id);
        return AjaxJson.getSuccess();
    }



    /**
     * 改
     */
    @RequestMapping("update")
    @SaCheckPermission(TbPersonVisitRecord.PERMISSION_CODE)
    public AjaxJson updateHealthRecord(TbPersonVisitRecord t) {
        tbPersonVisitRecordService.update(t);
        return AjaxJson.getSuccess();
    }
    /**
     * 健康码记录-改
     */
    @RequestMapping("updateHealthRecord")
    @SaCheckPermission(TbPersonVisitRecord.PERMISSION_CODE_HEALTH_EDIT)
    public AjaxJson update(TbPersonVisitRecord t) {
        tbPersonVisitRecordService.update(t);
        return AjaxJson.getSuccess();
    }

    /**
     * 查 - 根据id
     */
    @RequestMapping("getById")
    public AjaxJson getById(Long id) {
        TbPersonVisitRecord t = tbPersonVisitRecordService.getById(id);
        return AjaxJson.getSuccessData(t);
    }

    /**
     * 查集合 - 根据条件（参数为空时代表忽略指定条件）
     */
    @RequestMapping("getList")
    public AjaxJson getList() {
        SoMap so = SoMap.getRequestSoMap();
        if (!StpUserUtil.isSuperAdmin()) {
            so.put("deptId",StpUserUtil.getDeptId());
        }
        List<TbPersonVisitRecord> list = tbPersonVisitRecordService.getList(so.startPage());
        return AjaxJson.getPageData(so.getDataCount(), list);
    }
    @RequestMapping("export")
    @SaCheckPermission(TbPersonVisitRecord.PERMISSION_CODE_EXPORT)
    public AjaxJson export() {
        SoMap soMap = SoMap.getRequestSoMap();
        if (!StpUserUtil.isSuperAdmin()) {
            soMap.put("deptId",StpUserUtil.getDeptId());
        }
        return AjaxJson.getSuccessData(tbPersonVisitRecordService.export(soMap));
    }




}
