package com.pj.api;


import cn.dev33.satoken.annotation.SaCheckPermission;
import cn.dev33.satoken.stp.StpUtil;
import cn.hutool.core.util.StrUtil;
import com.pj.current.satoken.StpUserUtil;
import com.pj.project.re_mini_user_car_filing.ReMiniUserCarFiling;
import com.pj.project.re_mini_user_car_filing.ReMiniUserCarFilingService;
import com.pj.project.re_mini_user_person_filing.ReMiniUserPersonFiling;
import com.pj.project.re_mini_user_person_filing.ReMiniUserPersonFilingService;
import com.pj.project.sp_admin.SpAdminService;
import com.pj.project.tb_car_filing.TbCarFiling;
import com.pj.project.tb_car_filing.TbCarFilingService;
import com.pj.project.tb_mini_user.TbMiniUser;
import com.pj.project.tb_mini_user.TbMiniUserService;
import com.pj.project.tb_person_filing.TbPersonFiling;
import com.pj.project.tb_person_filing.TbPersonFilingService;
import com.pj.project.tb_person_visit_record.TbPersonVisitRecord;
import com.pj.project.tb_person_visit_record.TbPersonVisitRecordService;
import com.pj.project.wx.WxService;
import com.pj.utils.sg.AjaxJson;
import com.pj.utils.so.SoMap;
import lombok.extern.slf4j.Slf4j;
import org.aspectj.weaver.loadtime.Aj;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import java.util.List;
import java.util.stream.Collectors;

@RequestMapping("wx")
@RestController
@Slf4j
public class WxController {
    @Resource
    private WxService wxService;
    @Resource
    private TbMiniUserService tbMiniUserService;
    @Resource
    private TbPersonFilingService tbPersonFilingService;
    @Resource
    private ReMiniUserPersonFilingService reMiniUserPersonFilingService;
    @Resource
    private TbCarFilingService tbCarFilingService;
    @Resource
    private ReMiniUserCarFilingService reMiniUserCarFilingService;
    @Resource
    private TbPersonVisitRecordService tbPersonVisitRecordService;

    @Resource
    private SpAdminService spAdminService;


    @GetMapping(value = "getRedirectUrl")
    public AjaxJson getRedirectUrl(String path, @RequestParam(required = false) String state) {
        return AjaxJson.getSuccessData(wxService.getRedirectUrl(path, state));
    }

    @GetMapping(value = "getOpenidByCode")
    public AjaxJson getOpenidByCode(String code, @RequestParam(required = false) String openid) {
        return AjaxJson.getSuccessData(wxService.getOpenidByCode(code, openid));
    }

    @GetMapping(value = "getInfoByOpenid")
    public AjaxJson getInfoByOpenid(String openid) {
        TbMiniUser tbMiniUser = tbMiniUserService.findByOpenid(openid);
        if (tbMiniUser != null) {
            StpUtil.login(tbMiniUser.getSpAdminId());
            String tokenValue = StpUtil.getTokenValue();
            tbMiniUser.setToken(tokenValue);
            StpUserUtil.setDeptId(tbMiniUser.getDeptId() + "");
            StpUserUtil.setAdmin(tbMiniUser.getName(), tbMiniUser.getPhone());
            StpUserUtil.setMiniUserId(tbMiniUser.getId());
            StpUserUtil.setMiniUserType(tbMiniUser.getType());
            StpUserUtil.setAdminType(tbMiniUser.getSuperAdmin() + "");
        }
        return AjaxJson.getSuccessData(tbMiniUser);
    }

    @GetMapping(value = "getWxConfig")
    public AjaxJson getWxConfig(String url) {
        return AjaxJson.getSuccessData(wxService.getWxConfig(url));
    }

    @GetMapping(value = "person-filing")
    public AjaxJson getPersonFiling() {
        SoMap so = SoMap.getRequestSoMap();
        if (StpUserUtil.isInnerMiniUser()) {
            //内部人员 自己备案的+部门的备案
            so.put("loginId", StpUserUtil.getLoginIdAsLong());
            if (!StpUserUtil.isSuperAdmin()) {
                so.put("deptId", StpUserUtil.getDeptId());
            }
            List<TbPersonFiling> list = tbPersonFilingService.getForWx(so.startPage());
            return AjaxJson.getPageData(so.getDataCount(), list);
        } else {
            //备案自己的+帮别人备案的
            so.put("miniUserId", StpUserUtil.getMiniUserId());
            List<TbPersonFiling> list = tbPersonFilingService.getOuterList(so.startPage());
            return AjaxJson.getPageData(so.getDataCount(), list);
        }
    }

    @PostMapping("addPersonFiling")
    public AjaxJson addPersonFiling(TbPersonFiling tbPersonFiling) {
        if (!StpUserUtil.isInnerMiniUser()) {
            tbPersonFiling.setDeptId(StpUserUtil.getDeptId());
        }
        TbPersonFiling db = tbPersonFilingService.findByIdCard(tbPersonFiling.getIdCard());
        if (db!=null){
            tbPersonFiling=db;
        }else {
            tbPersonFilingService.add(tbPersonFiling);
        }
        ReMiniUserPersonFiling reMiniUserPersonFiling = new ReMiniUserPersonFiling();
        reMiniUserPersonFiling.setFilingPersonId(tbPersonFiling.getId())
                .setMiniUserId(StpUserUtil.getMiniUserId());
        reMiniUserPersonFilingService.save(reMiniUserPersonFiling);
        return AjaxJson.getSuccess();
    }

    @PostMapping("editPersonFiling")
    public AjaxJson editPersonFiling(TbPersonFiling tbPersonFiling) {
        tbPersonFilingService.update(tbPersonFiling);
        return AjaxJson.getSuccess();
    }

    @PostMapping("deletePersonFiling")
    public AjaxJson deletePersonFiling(Integer id) {
        tbPersonFilingService.delete(id);
        return AjaxJson.getSuccess();
    }

    @PostMapping("addCarFiling")
    public AjaxJson addCarFiling(TbCarFiling tbCarFiling) {
        if (!StpUserUtil.isInnerMiniUser()) {
            tbCarFiling.setDeptId(StpUserUtil.getDeptId());
        }
        tbCarFilingService.add(tbCarFiling);
        ReMiniUserCarFiling reMiniUserCarFiling = new ReMiniUserCarFiling();
        reMiniUserCarFiling.setFilingCarId(tbCarFiling.getId())
                .setMiniUserId(StpUserUtil.getMiniUserId());
        reMiniUserCarFilingService.save(reMiniUserCarFiling);
        return AjaxJson.getSuccess();
    }

    @GetMapping("car-filing")
    public AjaxJson getCarFiling() {
        SoMap so = SoMap.getRequestSoMap();
        if (StpUserUtil.isInnerMiniUser()) {
            //内部人员 自己备案的+部门的备案
            so.put("loginId", StpUserUtil.getLoginIdAsLong());
            if (!StpUserUtil.isSuperAdmin()) {
                so.put("deptId", StpUserUtil.getDeptId());
            }
            List<TbCarFiling> list = tbCarFilingService.getForWx(so.startPage());
            return AjaxJson.getPageData(so.getDataCount(), list);
        } else {
            //备案自己的+帮别人备案的
            so.put("miniUserId", StpUserUtil.getMiniUserId());
            List<TbCarFiling> list = tbCarFilingService.getOuterList(so.startPage());
            return AjaxJson.getPageData(so.getDataCount(), list);
        }


    }

    @PostMapping("deleteCarFiling")
    public AjaxJson deleteCarFiling(Long id) {
        tbCarFilingService.delete(id);
        return AjaxJson.getSuccess();
    }

    @PostMapping("loginByOpenid")
    public AjaxJson loginByOpenid() {
        SoMap soMap = SoMap.getRequestSoMap();
        String openid = soMap.getString("openid");
        if (StrUtil.isEmpty(openid)) {
            return AjaxJson.getError("参数异常");
        }
        return tbMiniUserService.loginByOpenid(openid);
    }

    @RequestMapping("solve")
    public AjaxJson solve() {
        SoMap soMap = SoMap.getRequestSoMap();
        tbPersonVisitRecordService.solve(soMap);
        return AjaxJson.getSuccess();
    }


    @PostMapping("bind")
    public AjaxJson bind(String openid, String key, String password, Integer type, String name) {
        return spAdminService.bindUser(openid, key, password, type, name);
    }
}
