package com.pj.current.netty.command.handler.impl;

import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.RandomUtil;
import cn.hutool.core.util.StrUtil;
import cn.hutool.json.JSONObject;
import cn.hutool.json.JSONUtil;
import com.pj.current.netty.command.constants.CommandType;
import com.pj.current.netty.command.handler.IHandler;
import com.pj.current.netty.command.handler.dto.WhitelistDTO;
import com.pj.current.netty.dto.OKMsg;
import com.pj.current.netty.dto.RequestPackageDTO;
import com.pj.current.task.TaskService;
import com.pj.project.tb_dept.TbDept;
import com.pj.project.tb_dept.TbDeptService;
import com.pj.project.tb_person_filing.TbPersonFiling;
import com.pj.project.tb_person_filing.TbPersonFilingService;
import com.pj.project.tb_person_filing.task.Base64ToImage;
import com.pj.project.tb_person_filing.task.DelayIsuued;
import com.pj.project.tb_terminal.TbTerminal;
import com.pj.project.tb_terminal.TbTerminalService;
import com.pj.project4sp.spcfg.SpCfgUtil;
import io.netty.channel.ChannelHandlerContext;
import lombok.extern.slf4j.Slf4j;
import org.springframework.stereotype.Service;

import javax.annotation.Resource;
import java.util.Date;
import java.util.List;

/**
 * 上传人员信息
 */
@Service
@Slf4j
public class UploadUserHandler implements IHandler {
    @Resource
    private TbTerminalService tbTerminalService;
    @Resource
    private TbPersonFilingService tbPersonFilingService;
    @Resource
    private TaskService taskService;

    @Resource
    private TbDeptService tbDeptService;


    @Override
    public CommandType commandType() {
        return CommandType.UPLOAD_USER;
    }

    @Override
    public void handler(ChannelHandlerContext ctx, RequestPackageDTO packageDTO) {
        log.info("设备上传人员信息:{}", JSONUtil.toJsonStr(packageDTO));
        JSONObject params = JSONUtil.parseObj(packageDTO.getPayload().getParams());
        String sn = params.getStr("sn");
        TbTerminal tbTerminal = tbTerminalService.findBySN(sn);
        if (tbTerminal == null) {
            log.error("========设备不存在=========:{}", sn);
            return;
        }
        tbTerminal.setLastOnLine(DateUtil.now());
        tbTerminalService.updateById(tbTerminal);
        ctx.write(OKMsg.ok());
        ctx.flush();
        boolean terminalFilling = SpCfgUtil.termianlFilling();
        if (!terminalFilling) {
            log.error("系统不允许设备端进行备案================");
            return;
        }
        WhitelistDTO uploadUserDTO = JSONUtil.toBean(params.getStr("users"), WhitelistDTO.class);
        String idCarNo=uploadUserDTO.getIdentity_number();
        TbPersonFiling personFiling = tbPersonFilingService.findByIdCard(idCarNo);
        //是否允许设备端备案
        if (personFiling == null) {
            TbPersonFiling tbPersonFiling = new TbPersonFiling();
            TbDept dept = tbDeptService.findTheDept(TbDept.AdminTypeEnum.TEMP_TYPE.getType());
            Date now = new Date();
            boolean needJudge = SpCfgUtil.fillingNeedJudge();
            tbPersonFiling.setCreateTime(now)
                    .setUpdateTime(now)
                    .setPin(uploadUserDTO.getPin())
                    .setCreateBy("设备上传").setDeptId(dept.getId()).setDeptName(dept.getName())
                    .setPin(idCarNo)
                    .setIdCard(idCarNo)
                    .setJudgeState(needJudge?0:1).setJudgeTime(needJudge?null:now).setJudgeBy(needJudge?"":"终端备案上传")
                    .setName(uploadUserDTO.getName());
            tbPersonFilingService.save(tbPersonFiling);
            //转换图片
            taskService.addTask(new Base64ToImage(RandomUtil.randomNumbers(10), 50, tbPersonFiling.getId(), uploadUserDTO.getPhoto()));
            //分发人员信息到其他终端
            taskService.addTask(new DelayIsuued(RandomUtil.randomNumbers(11), 4000, tbPersonFiling.getId(), sn));
        } else {
            if (!StrUtil.equals(personFiling.getPin(), uploadUserDTO.getPin())) {//更新pin
                personFiling.setPin(uploadUserDTO.getPin());
                tbPersonFilingService.updateById(personFiling);
                taskService.addTask(new DelayIsuued(RandomUtil.randomNumbers(10), 4000, personFiling.getId(), sn));
            }
        }
    }
}
