package com.pj.project.sp_admin;

import cn.hutool.core.util.PhoneUtil;
import cn.hutool.core.util.RandomUtil;
import cn.hutool.core.util.StrUtil;
import com.pj.current.config.SystemObject;
import com.pj.current.global.BusinessException;
import com.pj.project.tb_dept.TbDept;
import com.pj.project.tb_dept.TbDeptService;
import com.pj.project.tb_mini_user.TbMiniUser;
import com.pj.project.tb_mini_user.TbMiniUserService;
import com.pj.utils.sg.AjaxJson;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;

import com.pj.project4sp.SP;

import cn.dev33.satoken.stp.StpUtil;

import javax.annotation.Resource;
import java.util.Date;
import java.util.List;

/**
 * Service: admin管理员
 *
 * @author kong
 */
@Service
@Transactional(rollbackFor = Exception.class)
public class SpAdminService {


    @Resource
    SpAdminMapper spAdminMapper;

    @Resource
    SpAdminPasswordService spAdminPasswordService;
    @Resource
    private TbDeptService tbDeptService;

    @Resource
    private TbMiniUserService tbMiniUserService;


    /**
     * 管理员添加一个管理员
     *
     * @param admin
     * @return
     */
    public long add(SpAdmin admin) {
        String phone = admin.getPhone();
        if (!PhoneUtil.isPhone(phone)) {
            throw new BusinessException("请输入正确的手机号");
        }
        admin.setName(admin.getPhone());
        // 检查姓名是否合法
        SpAdminUtil.checkAdmin(admin);

        // 创建人，为当前账号
        admin.setCreateByAid(StpUtil.getLoginIdAsLong());
        // 开始添加
        handlerDept(admin, admin.getDeptId());
        spAdminMapper.add(admin);
        // 获取主键
        long id = SP.publicMapper.getPrimarykey();
        // 更改密码（md5与明文）
        spAdminPasswordService.updatePassword(id, admin.getPassword2());

        // 返回主键
        return id;
    }


    public void updateDept(Long id, Long deptId) {
        SpAdmin spAdmin = spAdminMapper.getById(id);
        handlerDept(spAdmin, deptId);
        spAdminMapper.updateDept(spAdmin);
    }

    private void handlerDept(SpAdmin spAdmin, Long deptId) {
        int superAdmin = 0;
        TbDept tbDept = tbDeptService.getById(deptId);
        if (TbDept.AdminTypeEnum.ADMIN_TYPE.getType().equals(tbDept.getAdminType())) {
            superAdmin = 1;
        }
        spAdmin.setDeptName(tbDept.getName()).setSuperAdmin(superAdmin);
    }

    public void removeByPhone(String phone) {
        spAdminMapper.removeByPhone(phone);
    }


    public AjaxJson bindUser(String openid, String key, String password, Integer type, String name) {
        TbMiniUser db = tbMiniUserService.findByOpenid(openid);
        if (db != null) {
            return AjaxJson.getError("您已绑定其他账户");
        }
        if (TbMiniUser.TypeEnum.INTERIOR_TYPE.getType().equals(type)) {
            SpAdmin spAdmin = spAdminMapper.getByName(key);
            if (spAdmin == null) {
                return AjaxJson.getError("账号错误");
            }
            String md5Password = SystemObject.getPasswordMd5(spAdmin.getId(), password);
            if (!spAdmin.getPassword2().equals(md5Password)) {
                return AjaxJson.getError("密码错误");
            }
            // 4、是否禁用
            if (spAdmin.getStatus() == 2) {
                return AjaxJson.getError("此账号已被禁用");
            }
            if (spAdmin.getBind() == 1) {
                return AjaxJson.getError("此账号已被占用");
            }
            spAdmin.setOpenid(openid);
            spAdminMapper.bindUser(spAdmin);
            TbMiniUser tbMiniUser = buildMiniUser(spAdmin, openid);
            tbMiniUserService.save(tbMiniUser);
            return AjaxJson.getSuccessData(tbMiniUser);
        } else {
            TbMiniUser tbMiniUser = createMiniUser(key, openid, name);
            return AjaxJson.getSuccessData(tbMiniUser);
        }
    }

    private TbMiniUser createMiniUser(String key, String openid, String name) {
       TbDept dept= tbDeptService.findTheDept(TbDept.AdminTypeEnum.TEMP_TYPE.getType());
        TbMiniUser tbMiniUser = new TbMiniUser();
        tbMiniUser.setCreateTime(new Date()).setSpAdminId(RandomUtil.randomLong(100000, 999999))
                .setOpenid(openid).setDeptId(dept.getId()+"").setSuperAdmin(0)
                .setPhone(key).setType(TbMiniUser.TypeEnum.TEMPORARY_TYPE.getType())
                .setName(name);
        tbMiniUserService.save(tbMiniUser);
        return tbMiniUser;
    }

    private TbMiniUser buildMiniUser(SpAdmin spAdmin, String openid) {
        TbMiniUser tbMiniUser = new TbMiniUser();
        tbMiniUser.setCreateTime(new Date()).setSuperAdmin(spAdmin.getSuperAdmin())
                .setDeptId(spAdmin.getDeptId()+"")
                .setOpenid(openid).setSpAdminId(spAdmin.getId())
                .setPhone(spAdmin.getPhone())
                .setType(2)
                .setName(spAdmin.getNickname());
        return tbMiniUser;
    }
}
