package com.pj.project.tb_manager;

import java.util.Date;
import java.util.List;

import cn.hutool.core.util.StrUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.service.IService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.pj.current.global.BusinessException;
import com.pj.current.satoken.StpUserUtil;
import com.pj.project.sp_admin.SpAdmin;
import com.pj.project.sp_admin.SpAdminService;
import com.pj.project.tb_dept.TbDept;
import com.pj.project.tb_dept.TbDeptService;
import com.pj.project.tb_mini_user.TbMiniUserService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import com.pj.utils.so.*;

import javax.annotation.Resource;

/**
 * Service: tb_manager -- 防疫人员
 *
 * @author qzyReal
 */
@Service
@Transactional(rollbackFor = Exception.class)
public class TbManagerService extends ServiceImpl<TbManagerMapper, TbManager> implements IService<TbManager> {

    /**
     * 底层 Mapper 对象
     */
    @Resource
    TbManagerMapper tbManagerMapper;

    @Resource
    private TbMiniUserService tbMiniUserService;


    /**
     * 增
     */
    public void add(TbManager t) {
        Date now = new Date();
        t.setCreateTime(now).setUpdateTime(now)
                .setCreateBy(StpUserUtil.getAdminName())
                .setUpdateBy(StpUserUtil.getAdminName());
        TbManager db = findByPhoneAndType(t.getPhone(),t.getAlarm());
        if (db != null) {
            throw new BusinessException("手机号被占用");
        }
        this.save(t);
    }

    public TbManager findByPhoneAndType(String phone,int type) {
        QueryWrapper<TbManager> ew = new QueryWrapper<>();
        ew.lambda().eq(TbManager::getPhone, phone).eq(TbManager::getAlarm,type);
        return getOne(ew);
    }

    /**
     * 删
     */
    public void delete(Long id) {
        TbManager tbManager = this.getById(id);
        tbMiniUserService.removeByPhone(tbManager.getPhone());
        this.removeById(id);
    }

    /**
     * 改
     */
    public void update(TbManager t) {
        t.setUpdateTime(new Date()).setUpdateBy(StpUserUtil.getAdminName());
        TbManager db = findByPhoneAndType(t.getPhone(),t.getAlarm());
        if (db != null && t.getId().longValue() != db.getId().longValue()) {
            throw new BusinessException("手机号被占用");
        }
        this.updateById(t);
    }

    /**
     * 查
     */
    public TbManager getById(Long id) {
        return super.getById(id);
    }

    /**
     * 查集合 - 根据条件（参数为空时代表忽略指定条件）
     */
    public List<TbManager> getList(SoMap so) {
        return tbManagerMapper.getList(so);
    }


    public List<TbManager> findAlarmPersonByVenuesId(Long venuesId) {
        return tbManagerMapper.findAlarmPersonByVenuesId(venuesId);

    }
}
