package com.pj.project.tb_terminal;

import java.util.Date;
import java.util.List;

import cn.dev33.satoken.stp.StpUtil;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.util.RandomUtil;
import cn.hutool.core.util.StrUtil;
import cn.hutool.json.JSONObject;
import cn.hutool.json.JSONUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.core.conditions.update.UpdateWrapper;
import com.baomidou.mybatisplus.extension.service.IService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.pj.current.config.MyConfig;
import com.pj.current.global.BusinessException;
import com.pj.current.netty.command.constants.CommandType;
import com.pj.current.netty.dto.RequestPackageDTO;
import com.pj.current.netty.heart.HeartBeatHandler;
import com.pj.current.satoken.StpUserUtil;
import com.pj.project.tb_command_log.TbCommandLog;
import com.pj.project.tb_command_log.TbCommandLogService;
import com.pj.project.tb_dept.TbDept;
import com.pj.project.tb_dept.TbDeptService;
import com.pj.project.tb_person_black.TbPersonBlack;
import com.pj.project.tb_person_black.TbPersonBlackService;
import com.pj.project.tb_person_filing.TbPersonFiling;
import com.pj.project.tb_person_filing.TbPersonFilingService;
import com.pj.project.tb_person_visit_record.TbPersonVisitRecord;
import com.pj.project.tb_terminal.bo.CheckBO;
import com.pj.project.tb_terminal.bo.CheckDataDTO;
import com.pj.project.tb_terminal.bo.MonitorDTO;
import com.pj.project.tb_venues.TbVenues;
import com.pj.project.tb_venues.TbVenuesService;
import com.pj.project4sp.spcfg.SpCfgUtil;
import com.pj.utils.cache.RedisUtil;
import com.pj.utils.sg.AjaxJson;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import com.pj.utils.so.*;

import javax.annotation.Resource;

/**
 * Service: tb_terminal -- 设备管理
 *
 * @author qzyReal
 */
@Service
@Transactional(rollbackFor = Exception.class)
@Slf4j
public class TbTerminalService extends ServiceImpl<TbTerminalMapper, TbTerminal> implements IService<TbTerminal> {

    /**
     * 底层 Mapper 对象
     */
    @Autowired
    TbTerminalMapper tbTerminalMapper;
    @Resource
    private MyConfig myConfig;

    @Resource
    private TbCommandLogService tbCommandLogService;

    @Resource
    private TbVenuesService tbVenuesService;

    @Resource
    private TbPersonFilingService tbPersonFilingService;

    @Resource
    private TbPersonBlackService tbPersonBlackService;
    @Resource
    private TbDeptService tbDeptService;


    /**
     * 增
     */
    public void add(TbTerminal t) {
        this.save(t);
    }

    /**
     * 删
     */
    public void delete(Long id) {
        this.removeById(id);
    }

    /**
     * 改
     */
    public void update(TbTerminal t) {
        Long channelId = t.getChannelId();
        TbVenues tbVenues = tbVenuesService.getById(channelId);
        t.setChannelName(tbVenues.getName())
                .setUpdateBy(StpUtil.getLoginIdAsString()).setUpdateTime(new Date());
        TbTerminal db = this.getById(t.getId());
        this.updateById(t);
    }


    /**
     * 查
     */
    TbTerminal getById(Long id) {
        return super.getById(id);
    }

    /**
     * 查集合 - 根据条件（参数为空时代表忽略指定条件）
     */
    public List<TbTerminal> getList(SoMap so) {
        return tbTerminalMapper.getList(so);
    }

    /**
     * 初始化设备
     *
     * @param sn
     * @param clientIp
     */
    public void initTerminal(String sn, String clientIp) {
        TbTerminal tbTerminal = findBySN(sn);
        if (tbTerminal == null) {
            tbTerminal = new TbTerminal();
            tbTerminal.setIpAddress(clientIp).setState(TbTerminal.StateType.ONLINE.getState())
                    .setSn(sn).setCreateTime(new Date());
            this.save(tbTerminal);
            tbPersonFilingService.issued(tbTerminal);
            tbPersonBlackService.issued(tbTerminal);
            this.startHeartCommand(tbTerminal.getSn());
            return;
        }
        if (!StrUtil.equals(tbTerminal.getState(), TbTerminal.StateType.ONLINE.getState())) {
            tbTerminal.setState(TbTerminal.StateType.ONLINE.getState());
            this.updateById(tbTerminal);
        }
    }

    /**
     * 根据sn查找
     *
     * @param sn
     * @return
     */
    public TbTerminal findBySN(String sn) {
        QueryWrapper<TbTerminal> ew = new QueryWrapper<>();
        ew.eq("sn", sn);
        return getOne(ew);
    }

    /**
     * 重启
     *
     * @param sn
     */
    public void restart(String sn) {
        if (HeartBeatHandler.STATE_CACHE.get(sn) == null) {
            throw new BusinessException("设备已离线");
        }
        TbTerminal tbTerminal = this.findBySN(sn);
        //缓存命令
        setCommand(CommandType.ISSUED_REBOOT_COMMAND.getFuncId(), null, tbTerminal);
        //心跳缓存
        startHeartCommand(sn);
    }

    /**
     * 设置命令
     *
     * @param funcId func id
     * @param params 具体参数
     * @return
     */
    public void setCommand(String funcId, Object params, TbTerminal tbTerminal) {
        String sn = tbTerminal.getSn();
        String msgId = RandomUtil.randomNumbers(8);
        String key = myConfig.getCommandPrefix() + sn;
        String commandStr = RedisUtil.get(key);
        List<RequestPackageDTO> commandList = JSONUtil.toList(commandStr, RequestPackageDTO.class);
        RequestPackageDTO commandDTO = new RequestPackageDTO();
        RequestPackageDTO.PayloadBean payloadBean = new RequestPackageDTO.PayloadBean();

        payloadBean.setParams(params);
        commandDTO.setFuncId(funcId).setPayload(payloadBean)
                .setMsgId(msgId);
        commandList.add(commandDTO);
        TbCommandLog tbCommandLog = new TbCommandLog();
        tbCommandLog.setChannelName(tbTerminal.getChannelName()).setCommand(JSONUtil.toJsonStr(commandDTO))
                .setMsgId(msgId).setSn(sn)
                .setTerminalName(tbTerminal.getName()).setCreateTime(new Date()).setCreateBy(StpUserUtil.getAdminName());
        //日志记录
        tbCommandLogService.save(tbCommandLog);
        RedisUtil.set(myConfig.getCommandPrefix() + sn, JSONUtil.toJsonStr(commandList));
    }

    /**
     * 查看设备属性,不记录日志
     *
     * @param funcId
     * @param sn
     */
    public void checkOptions(String funcId, String sn) {
        String msgId = RandomUtil.randomNumbers(8);
        String key = myConfig.getCommandPrefix() + sn;
        String commandStr = RedisUtil.get(key);
        List<RequestPackageDTO> commandList = JSONUtil.toList(commandStr, RequestPackageDTO.class);
        RequestPackageDTO commandDTO = new RequestPackageDTO();
        commandDTO.setFuncId(funcId)
                .setMsgId(msgId);
        commandList.add(commandDTO);
        RedisUtil.set(myConfig.getCommandPrefix() + sn, JSONUtil.toJsonStr(commandList));
    }

    /**
     * 设置心跳command=1,设备开始请求命令服务器接口
     *
     * @param sn sn码
     */
    public void startHeartCommand(String sn) {
        log.info("设置心跳命令===========:{}", sn);
        RequestPackageDTO requestPackageDTO = RequestPackageDTO.getHeartCommandDTO();
        RedisUtil.set(myConfig.getHeartPrefix() + sn, JSONUtil.toJsonStr(requestPackageDTO));
    }

    /**
     * 解绑设备
     *
     * @param id
     */
    public void unbind(Long id) {
        TbTerminal tbTerminal = this.getById(id);
        if (StrUtil.equals(tbTerminal.getState(), TbTerminal.StateType.OFFLINE.getState())) {
            throw new BusinessException("设备已离线");
        }
        tbTerminalMapper.unbind(id);
    }

    /**
     * 清除数据
     *
     * @param id
     */
    public void clearData(Long id) {
        TbTerminal tbTerminal = this.getById(id);
        if (StrUtil.equals(tbTerminal.getState(), TbTerminal.StateType.OFFLINE.getState())) {
            throw new BusinessException("设备已离线");
        }
        this.clearData(tbTerminal);
    }

    /**
     * 将离线设备设置为下线
     *
     * @param snList
     */
    public void offlineTerminal(List<String> snList) {
        UpdateWrapper<TbTerminal> ew = new UpdateWrapper<>();
        ew.notIn(!snList.isEmpty(), "sn", snList);
        ew.setSql("state='" + TbTerminal.StateType.OFFLINE.getState() + "'");
        this.update(ew);
    }


    /**
     * 清空所有数据
     *
     * @param tbTerminal
     */
    public void clearData(TbTerminal tbTerminal) {
        setCommand(CommandType.ISSUED_CLEAR_COMMAND.getFuncId(), null, tbTerminal);
        startHeartCommand(tbTerminal.getSn());
    }

    /**
     * 远程开门
     *
     * @param id
     */
    public void open(Long id) {
        log.info("==========远程开闸============");
        TbTerminal tbTerminal = this.getById(id);
        if (StrUtil.equals(tbTerminal.getState(), TbTerminal.StateType.OFFLINE.getState())) {
            throw new BusinessException("设备已离线");
        }
        setCommand(CommandType.ISSUED_NORMALOPEN_COMMAND.getFuncId(), null, tbTerminal);
        startHeartCommand(tbTerminal.getSn());
    }

    /**
     * 同步时间
     *
     * @param id
     */
    public void asyncTime(Long id) {
        TbTerminal tbTerminal = this.getById(id);
        if (StrUtil.equals(tbTerminal.getState(), TbTerminal.StateType.OFFLINE.getState())) {
            throw new BusinessException("设备已离线");
        }
        JSONObject params = new JSONObject();
        params.set("time", System.currentTimeMillis() / 1000);
        setCommand(CommandType.SYNC_TIME_COMMAND.getFuncId(), params, tbTerminal);
        startHeartCommand(tbTerminal.getSn());

    }

    public List<TbTerminal> findByChannelId(Long channelId) {
        QueryWrapper<TbTerminal> ew = new QueryWrapper<>();
        ew.eq("channel_id", channelId);
        return list(ew);
    }

    public List<MonitorDTO> getMonitor() {
        return tbTerminalMapper.getMonitor();
    }

    /**
     * 终端请求是否放行
     *
     * @param checkBO
     * @return
     */
    public AjaxJson check(CheckBO checkBO) {
        log.info("设备请求放行接口:{}", JSONUtil.toJsonStr(checkBO));
        int acidLimit = SpCfgUtil.getAcidLimit();
        int tourLimit = SpCfgUtil.getTourLimit();
        String now = DateUtil.now();
        String idCard = checkBO.getIdentity_number();
        CheckDataDTO checkDataDTO = new CheckDataDTO();
        checkDataDTO.setNucleic_restrict(acidLimit)
                .setTrip_restrict(tourLimit).setIs_pass(0);
        //是否备案、备案是否通过审核
        TbPersonFiling tbPersonFiling = tbPersonFilingService.findByIdCard(idCard);
        if (tbPersonFiling == null || tbPersonFiling.getJudgeState() == 0) {
            log.info("返回信息:{},{}", "人员未备案", JSONUtil.toJsonStr(checkDataDTO));
            return AjaxJson.getSuccess("人员未备案", checkDataDTO);
        }
        TbDept tbDept = tbDeptService.getById(tbPersonFiling.getDeptId());
        String msg = tbDept.getName() + tbPersonFiling.getName();
        //体温是否异常
        double minTemperature = SpCfgUtil.getMinTemperature();
        if (minTemperature < checkBO.getTemperature()) {
            log.info("返回信息体温过高:{},{}", msg, JSONUtil.toJsonStr(checkDataDTO));
            return AjaxJson.getSuccess(msg, checkDataDTO);
        }
        //健康码是否绿码
        if (!StrUtil.equals(TbPersonVisitRecord.HealthEnum.GREEN.getStatus(), checkBO.getHealth_status())) {
            log.info("返回信息健康码异常:{},{}", msg, JSONUtil.toJsonStr(checkDataDTO));
            return AjaxJson.getSuccess(msg, checkDataDTO);
        }
        //黑名单用户
        TbPersonBlack black = tbPersonBlackService.findByIdCardAndTime(idCard, now);
        if (black != null) {
            log.info("返回信息黑名单:{},{}", msg, JSONUtil.toJsonStr(checkDataDTO));
            return AjaxJson.getSuccess(msg, checkDataDTO);
        }
        //通道是否开放
        TbTerminal tbTerminal = this.findBySN(checkBO.getSn());
        if (tbTerminal==null){
            log.info("返回信息非法设备:{},{}", msg, JSONUtil.toJsonStr(checkDataDTO));
            return AjaxJson.getSuccess("非法设备", checkDataDTO);
        }
        TbVenues tbVenues = tbVenuesService.getById(tbTerminal.getChannelId());
        if (tbVenues.getStatus()==0){
            log.info("返回信息通道关闭放行:{},{}", msg, JSONUtil.toJsonStr(checkDataDTO));
            return AjaxJson.getSuccess(msg, checkDataDTO);
        }
        checkDataDTO.setIs_pass(1);
        return AjaxJson.getSuccess(msg, checkDataDTO);
    }

}
